/* pen.hh
* This file is part of ncursesxx library
* Copyright (c) 2003,2004 by Stanislav Ievlev
*
* This file is covered by the GNU Library General Public License,
* which should be included with libncursesxx as the file COPYING.
*/
#ifndef NCURSESXX_WPEN_HH__
#define NCURSESXX_WPEN_HH__

#include <ncurses/basic_pen.hh>
#include <ncurses/assert.hh>

namespace ncurses
{
	template <>
	struct char_traits<wchar_t> /**< specialization for the wide strings */
	{
		typedef cchar_t*	char_type; /**< main internal character type */
		static
		void write(WINDOW *win, const wchar_t *str, int len)
		{
			::waddnwstr(win, str, len);
		}
		static
		void write(WINDOW *win, const char_type ch)
		{
			::wadd_wch(win, ch);
		}
		static
		void frame(WINDOW *win)
		{
			NCURSES_ASSERT(box_set(win, WACS_VLINE, WACS_HLINE));
		}
		static
		void h_line(WINDOW *win, const char_type ch, int n)
		{
			::whline_set(win, ch, n);
		}
		static
		void v_line(WINDOW *win, const char_type ch, int n)
		{
			::wvline_set(win, ch, n);
		}
	};

	template <>
	struct acs<wchar_t>
	{
		typedef char_traits<wchar_t>::char_type char_type;

		static const char_type ulcorner() { return WACS_ULCORNER; }
		static const char_type llcorner() { return WACS_LLCORNER; }
		static const char_type urcorner() { return WACS_URCORNER; }
		static const char_type lrcorner() { return WACS_LRCORNER; }
		static const char_type h_line() { return WACS_HLINE; }
		static const char_type v_line() { return WACS_VLINE; }
	};

	typedef basic_pen<wchar_t>	wpen;
}
#endif
